using System;
using System.IO;
using System.Text;
using System.Threading;
using skyscraper5.Dvb.Descriptors;
using skyscraper5.Mpeg2;
using skyscraper5.Skyscraper.Scraper;

namespace skyscraper5.Scorcher
{
	internal class DummyTsGenerator
	{
		public DummyTsGenerator()
		{
			fi = new FileInfo(string.Format("{0}.ts", DateTime.Now.Ticks));
			outputStream = fi.OpenWrite();
			tsGeneratorStream = new TsGeneratorStream();
		}

		private FileInfo fi;
		private FileStream outputStream;
		private TsGeneratorStream tsGeneratorStream;

		public void Run()
		{
			byte[] buffer = new byte[188];
			
			Random rng = new Random();
			tsGeneratorStream.TransportStreamId = (0x0111);
			PmtGenerator prog1 = tsGeneratorStream.AddProgram(0x0222, 0x0333);
			prog1.AddDescriptor(new PrivateDataSpecifierDescriptor(0x534f5048));
			PmtGenerator prog2 = tsGeneratorStream.AddProgram(0x0444, 0x0555);
			prog2.AddDescriptor(new PrivateDataSpecifierDescriptor(0x4f504849));
			PmtGenerator prog3 = tsGeneratorStream.AddProgram(0x0666, 0x0777);
			prog3.AddDescriptor(new PrivateDataSpecifierDescriptor(0x50484941));
			PmtGeneratorStream stream1 = prog3.AddStream(0x80, 0x0888);
			stream1.AddDescriptor(new PrivateDataSpecifierDescriptor(0x53504831));
			PmtGeneratorStream stream2 = prog3.AddStream(0x81, 0x0999);
			stream2.AddDescriptor(new PrivateDataSpecifierDescriptor(0x53504832));

			tsGeneratorStream.AddCaSystem(0x534f, 0x0bbb, Encoding.ASCII.GetBytes("Sophia"));
			tsGeneratorStream.GenerateTdt = true;
			tsGeneratorStream.GenerateTot = false;
			tsGeneratorStream.OriginalNetworkId = 0x0ccc;

			SdtGeneratorService sdt1 = tsGeneratorStream.AddService(0x0222);
			sdt1.AddName("Skyscraper Test Service 1", "sophia.net", ServiceDescriptor.ServiceTypeCoding.DataBroadcast);
			sdt1.AddDescriptor(new PrivateDataSpecifierDescriptor(0x53504831));
			SdtGeneratorService sdt2 = tsGeneratorStream.AddService(0x0444);
			sdt2.AddName("Skyscraper Test Service 2", "sophia.net", ServiceDescriptor.ServiceTypeCoding.DataBroadcast);
			sdt2.AddDescriptor(new PrivateDataSpecifierDescriptor(0x53504832));
			SdtGeneratorService sdt3 = tsGeneratorStream.AddService(0x0666);
			sdt3.AddName("Skyscraper Test Service 3", "sophia.net", ServiceDescriptor.ServiceTypeCoding.DataBroadcast);
			sdt3.AddDescriptor(new PrivateDataSpecifierDescriptor(0x53504833));

			tsGeneratorStream.Write(Encoding.UTF8.GetBytes("Weit hinten, hinter den Wortbergen, fern der Länder Vokalien und Konsonantien leben die Blindtexte. Abgeschieden wohnen sie in Buchstabhausen an der Küste des Semantik, eines großen Sprachozeans. Ein kleines Bächlein namens Duden fließt durch ihren Ort und versorgt sie mit den nötigen Regelialien. Es ist ein paradiesmatisches Land, in dem einem gebratene Satzteile in den Mund fliegen. Nicht einmal von der allmächtigen Interpunktion werden die Blindtexte beherrscht – ein geradezu unorthographisches Leben. Eines Tages aber beschloß eine kleine Zeile Blindtext, ihr Name war Lorem Ipsum, hinaus zu gehen in die weite Grammatik. Der große Oxmox riet ihr davon ab, da es dort wimmele von bösen Kommata, wilden Fragezeichen und hinterhältigen Semikoli, doch das Blindtextchen ließ sich nicht beirren. Es packte seine sieben Versalien, schob sich sein Initial in den Gürtel und machte sich auf den Weg. Als es die ersten Hügel des Kursivgebirges erklommen hatte, warf es einen letzten Blick zurück auf die Skyline seiner Heimatstadt Buchstabhausen, die Headline von Alphabetdorf und die Subline seiner eigenen Straße, der Zeilengasse. Wehmütig lief ihm eine rhetorische Frage über die Wange, dann setzte es seinen Weg fort. Unterwegs traf es eine Copy. Die Copy warnte das Blindtextchen, da, wo sie herkäme wäre sie zigmal umgeschrieben worden und alles, was von ihrem Ursprung noch übrig wäre, sei das Wort \"und\" und das Blindtextchen solle umkehren und wieder in sein eigenes, sicheres Land zurückkehren. Doch alles Gutzureden konnte es nicht überzeugen und so dauerte es nicht lange, bis ihm ein paar heimtückische Werbetexter auflauerten, es mit Longe und Parole betrunken machten und es dann in ihre Agentur schleppten, wo sie es für ihre Projekte wieder und wieder mißbrauchten. Und wenn es nicht umgeschrieben wurde, dann benutzen Sie es immernoch."));

			for (int i = 0; i < 20; i++)
			{
				for (int j = 0; j < 100; j++)
				{
					tsGeneratorStream.Read(buffer, 0, 188);
					outputStream.Write(buffer, 0, 188);
				}

				Thread.Sleep(1000);
			}

			outputStream.Flush();
			outputStream.Close();

			outputStream = fi.OpenRead();
			TsContext tsContext = new TsContext();
			SkyscraperContext sc = new SkyscraperContext(tsContext, null, null,0);
			sc.IngestFromStream(outputStream);
			outputStream.Close();

		}
	}
}
